unit Config;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, IniFiles;

type
  TConfigForm = class(TForm)
    BtnApply: TButton;
    BtnCancel: TButton;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    ChkReadOnly: TCheckBox;
    ChkHidden: TCheckBox;
    ChkSysFile: TCheckBox;
    GroupBox3: TGroupBox;
    MemIgnoreList: TMemo;
    EditSample: TEdit;
    UpdnSample: TUpDown;
    BtnMB: TButton;
    BtnKB: TButton;
    Label1: TLabel;
    EditLowerSize: TEdit;
    GroupBox4: TGroupBox;
    MemIgnoreExt: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BtnApplyClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure EditSampleExit(Sender: TObject);
    procedure BtnMBClick(Sender: TObject);
    procedure BtnKBClick(Sender: TObject);
    procedure EditLowerSizeExit(Sender: TObject);
  private

  public

  end;

type
    TConfig = class
    private
        FReadOnly: Boolean;
        FHidden: Boolean;
        FSysFile: Boolean;
        FIgnoreList: TStringList;
        FIgnoreExt: TStringList;
        FLowerSize: Integer;
    private
        procedure SetForm();
        procedure GetForm();
    public
        constructor Create();
        destructor Destroy(); override;
        function CheckFile(const AFileName: String): Boolean;
        procedure LoadConfig(const AIniPath: String);
        procedure SaveConfig(const AIniPath: String);
    end;

var
    ConfigForm: TConfigForm;
    GConfig: TConfig = nil;

//  t@CTCY擾(FileSize)
function FileSizeByFileSize(const FileName: string): Integer;

implementation

{$R *.dfm}

//=============================================================================
//  Utility
//=============================================================================

//-----------------------------------------------------------------------------
//  t@CTCY擾(FileSize)
function FileSizeByFileSize(const FileName: string): Integer;
var
    F: file;                            //  ^Ȃt@C
begin
    Result := -1;
    try
        AssignFile(F, FileName);
        try
            Reset(F, 1);                //  oCgt@C
            Result := FileSize(F);
        finally
            CloseFile(F);
        end;
    except
    end;
end;

//=============================================================================
//  TConfig implementation
//=============================================================================

//-----------------------------------------------------------------------------
//  RXgN^
constructor TConfig.Create();
begin
    FReadOnly := False;
    FHidden := True;
    FSysFile := True;

    FIgnoreList := THashedStringList.Create();
    FIgnoreList.Sorted := True;
    FIgnoreList.Duplicates := dupIgnore;
    FIgnoreList.CaseSensitive := False;

    //  [2008/01/19]
    FIgnoreExt := THashedStringList.Create();
    FIgnoreExt.Sorted := True;
    FIgnoreExt.Duplicates := dupIgnore;
    FIgnoreExt.CaseSensitive := False;

    FLowerSize := 500 * 1024;
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TConfig.Destroy();
begin
    FIgnoreList.Free();
    FIgnoreExt.Free();
end;

//-----------------------------------------------------------------------------
//  t@C󂯕t邩ǂ`FbN
function TConfig.CheckFile(const AFileName: String): Boolean;
var
    LAttr: Integer;
    LSize: Integer;
    LFileName: String;
    LFileExt: String;
begin
    Result := False;
    //  擾
    LAttr := SysUtils.FileGetAttr(AFileName);
    //  `FbN
    if (FReadOnly and ((LAttr and SysUtils.faReadOnly) <> 0)) then Exit;
    if (FHidden and ((LAttr and SysUtils.faHidden) <> 0)) then Exit;
    if (FSysFile and ((LAttr and SysUtils.faSysFile) <> 0)) then Exit;

    //  t@CTCY擾
    LSize := FileSizeByFileSize(AFileName);
    //  t@CTCY`FbN
    if (FLowerSize > LSize) then Exit;

    //  gq`FbN[2008/01/19]
    LFileExt := ExtractFileExt(AFileName);
    if (FIgnoreExt.IndexOf(LFileExt) <> -1) then Exit;

    //  t@C`FbN
    LFileName := ExtractFileName(AFileName);
    if (FIgnoreList.IndexOf(LFileName) <> -1) then Exit;

    //  I[O[
    Result := True;
end;

//-----------------------------------------------------------------------------
//  ݒe\
procedure TConfig.SetForm();
begin
    ConfigForm.ChkReadOnly.Checked := FReadOnly;    
    ConfigForm.ChkHidden.Checked := FHidden;
    ConfigForm.ChkSysFile.Checked := FSysFile;

    ConfigForm.MemIgnoreList.Text := Trim(FIgnoreList.Text);

    //  [2008/01/19]
    ConfigForm.MemIgnoreExt.Text := Trim(FIgnoreExt.Text);

    ConfigForm.EditLowerSize.Text := IntToStr(FLowerSize);
    ConfigForm.EditLowerSize.Tag := FLowerSize;
end;

//-----------------------------------------------------------------------------
//  ͓e
procedure TConfig.GetForm();
var
    i: Integer;
begin
    FReadOnly := ConfigForm.ChkReadOnly.Checked;
    FHidden := ConfigForm.ChkHidden.Checked;
    FSysFile := ConfigForm.ChkSysFile.Checked;

    for i := 0 to ConfigForm.MemIgnoreList.Lines.Count - 1 do
    begin
        FIgnoreList.Add(Trim(ConfigForm.MemIgnoreList.Lines[i]));
    end;

    //  [2008/01/19]
    for i := 0 to ConfigForm.MemIgnoreExt.Lines.Count - 1 do
    begin
        FIgnoreExt.Add(Trim(ConfigForm.MemIgnoreExt.Lines[i]));
    end;

    FLowerSize := ConfigForm.EditLowerSize.Tag;
end;

//-----------------------------------------------------------------------------
//  ݒǍ
procedure TConfig.LoadConfig(const AIniPath: String);
var
    LIniFile: TIniFile;
    LInt: Integer;
    i: Integer;
begin
    LIniFile := TIniFile.Create(AIniPath);
    try
        //  鑮ǂݍ
        FReadOnly := LIniFile.ReadBool('IgnoreAttr', 'FReadOnly', FReadOnly);
        FHidden := LIniFile.ReadBool('IgnoreAttr', 'FHidden', FHidden);
        FSysFile := LIniFile.ReadBool('IgnoreAttr', 'FSysFile', FSysFile);

        //  t@Cǂݍ
        LInt := LIniFile.ReadInteger('IgnoreList', 'ListCount', 0);
        for i := 0 to LInt - 1 do
        begin
            FIgnoreList.Add(LIniFile.ReadString('IgnoreList', IntToStr(i), ''));
        end;

        //  gqǂݍ[2008/01/19]
        LInt := LIniFile.ReadInteger('IgnoreExt', 'ListCount', 0);
        for i := 0 to LInt - 1 do
        begin
            FIgnoreExt.Add(LIniFile.ReadString('IgnoreExt', IntToStr(i), ''));
        end;

        //  TCYǂݍ
        FLowerSize := LIniFile.ReadInteger('LowerSize', 'FLowerSize', FLowerSize)
    finally
        LIniFile.Free();
    end;
end;

//-----------------------------------------------------------------------------
//  ݒۑ
procedure TConfig.SaveConfig(const AIniPath: String);
var
    LIniFile: TIniFile;
    i: Integer;
begin
    LIniFile := TIniFile.Create(AIniPath);
    try
        //  鑮ۑ
        LIniFile.WriteBool('IgnoreAttr', 'FReadOnly', FReadOnly);
        LIniFile.WriteBool('IgnoreAttr', 'FHidden', FHidden);
        LIniFile.WriteBool('IgnoreAttr', 'FSysFile', FSysFile);

        //  t@Cۑ
        LIniFile.WriteInteger('IgnoreList', 'ListCount', FIgnoreList.Count);
        for i := 0 to FIgnoreList.Count - 1 do
        begin
            LIniFile.WriteString('IgnoreList', IntToStr(i), FIgnoreList[i]);
        end;

        //  gqۑ[2008/01/19]
        LIniFile.WriteInteger('IgnoreExt', 'ListCount', FIgnoreExt.Count);
        for i := 0 to FIgnoreExt.Count - 1 do
        begin
            LIniFile.WriteString('IgnoreExt', IntToStr(i), FIgnoreExt[i]);
        end;

        //  TCYۑ
        LIniFile.WriteInteger('LowerSize', 'FLowerSize', FLowerSize)
    finally
        LIniFile.Free();
    end;
end;


//=============================================================================
//  TConfigForm implementation
//=============================================================================

//-----------------------------------------------------------------------------
//  tH[쐬
procedure TConfigForm.FormCreate(Sender: TObject);
begin
    //
end;

//-----------------------------------------------------------------------------
//  tH[j
procedure TConfigForm.FormDestroy(Sender: TObject);
begin
    //
end;

//-----------------------------------------------------------------------------
//  tH[\
procedure TConfigForm.FormShow(Sender: TObject);
begin
    GConfig.SetForm();
end;

//-----------------------------------------------------------------------------
//  Kp
procedure TConfigForm.BtnApplyClick(Sender: TObject);
begin
    GConfig.GetForm();
    Self.Close();
end;

//-----------------------------------------------------------------------------
//  LZ
procedure TConfigForm.BtnCancelClick(Sender: TObject);
begin
    Self.Close();
end;

//-----------------------------------------------------------------------------
//  TCYl̓`FbN
procedure TConfigForm.EditLowerSizeExit(Sender: TObject);
var
    LEdit: TEdit;
begin
    LEdit := TEdit(Sender);
    if (StrToIntDef(LEdit.Text, -1) < 0) then
    begin
        LEdit.Text := IntToStr(LEdit.Tag);
    end;
    LEdit.Tag := StrToIntDef(LEdit.Text, 0);
end;

//-----------------------------------------------------------------------------
//  Tv{bNX̓`FbN
procedure TConfigForm.EditSampleExit(Sender: TObject);
var
    LEdit: TEdit;
begin
    LEdit := TEdit(Sender);
    if (StrToIntDef(LEdit.Text, -1) < 0) then
    begin
        LEdit.Text := IntToStr(UpdnSample.Position);
    end;
end;

//-----------------------------------------------------------------------------
//  Tv(KB)
procedure TConfigForm.BtnKBClick(Sender: TObject);
begin
    EditLowerSize.Tag := UpdnSample.Position * 1024;
    EditLowerSize.Text := IntToStr(EditLowerSize.Tag);
end;

//-----------------------------------------------------------------------------
//  Tv(MB)
procedure TConfigForm.BtnMBClick(Sender: TObject);
begin
    EditLowerSize.Tag := UpdnSample.Position * 1024 * 1024;
    EditLowerSize.Text := IntToStr(EditLowerSize.Tag);
end;

end.
